/*******************************************************************************
 *
 * Copyright (C) 2016 - 2020 Xilinx, Inc.  All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * XILINX  BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF
 * OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 * Except as contained in this notice, the name of the Xilinx shall not be used
 * in advertising or otherwise to promote the sale, use or other dealings in
 * this Software without prior written authorization from Xilinx.
 *
 ******************************************************************************/
#ifndef _USR_PARAMS_H_
#define _USR_PARAMS_H_
/****************************** Include Files *********************************/
#include "xsd_fec_dec_docsis_short_params.h"
#include "xsd_fec_dec_docsis_medium_params.h"
#include "xsd_fec_dec_docsis_long_params.h"
#include "xsd_fec_enc_docsis_short_params.h"
#include "xsd_fec_enc_docsis_medium_params.h"
#include "xsd_fec_enc_docsis_long_params.h"

/*************************** Constant Definitions *****************************/
/* Constants used to access devices other than the sd-fec */
#define NUM_LDPC_CODES		(3UL)
#define MAX_ERR_MSG_SIZE	(256UL)
#define FEC_DEC			"/dev/xsdfec0"
#define FEC_ENC			"/dev/xsdfec1"

/*************************** Macro Definitions ********************************/

/* Informative message */
#define info_msg(...) printf(__VA_ARGS__)

extern char err_msg[MAX_ERR_MSG_SIZE];
extern user_params* enc_codes[NUM_LDPC_CODES];
extern user_params* dec_codes[NUM_LDPC_CODES];

#endif
